#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_complex.h>
#include <gsl/gsl_complex_math.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_sort.h>

/* calculate integrand of the definition of Levy alpha stable dist. */
gsl_complex levy (double *k, void *params)
{
    double t;
    double *par = (double *)params;
    double c, alpha, beta, x;
    gsl_complex val, arg;

    t = k[0];
    c = par[0]; alpha = par[1]; beta = par[2];
    x = par[3];

    GSL_SET_COMPLEX(&arg, 0, 1);
    arg = gsl_complex_mul_real(arg, -1.0);
    arg = gsl_complex_mul_real(arg, t);
    arg = gsl_complex_mul_real(arg, x);
    arg = gsl_complex_sub_real(arg, pow(fabs(c*t), alpha));
    val = gsl_complex_exp(arg);

    return val;
}

/* calculate integrand of the definition of Levy skew alpha stable dist. */
gsl_complex skew (double *k, void *params)
{
    double t;
    double *par = (double *)params;
    double c, alpha, beta, x;
    gsl_complex val, arg, arg2;

    t = k[0];
    c = par[0]; alpha = par[1]; beta = par[2];
    x = par[3];

    GSL_SET_COMPLEX(&arg2, 0, 1);
    if (alpha != 1.0) arg2 = gsl_complex_mul_real(arg2, tan(M_PI*alpha/2.0));
    else              arg2 = gsl_complex_mul_real(arg2, -2.0*log(fabs(t))/M_PI);
    arg2 = gsl_complex_mul_real(arg2, -beta*GSL_SIGN(t));
    arg2 = gsl_complex_add_real(arg2, 1.0);
    arg2 = gsl_complex_mul_real(arg2, pow(fabs(c*t), alpha));
    GSL_SET_COMPLEX(&arg, 0, 1);
    arg = gsl_complex_mul_real(arg, -1.0);
    arg = gsl_complex_mul_real(arg, t);
    arg = gsl_complex_mul_real(arg, x);
    arg = gsl_complex_sub(arg, arg2);
    val = gsl_complex_exp(arg);

    return val;
}

/*
calculation of PDFs

   "pseudo numerical integration" gives the PDF value while true numerical
   integration gives the CDF, because pseudo method doesn't multiply step size,
   it nearly equals to numerical differentiation of the CDF.
*/
void printhead1(void );
void printhead2(void );
int main (void)
{
    int i;
    double t, x;
    gsl_complex vsym, vsym2, vskew;
    double step = 0.001;    /* step for numerical integration */
    double R = 100.0;       /* integral region: instead of the infinity */
    double param[4] = { 1.0, 1.0, 1.0, 0 }; /* c, alpha, beta, x */
    double x0 = -5.0,      /* plot region */
           x1 =  5.0,
           h  =  0.1;      /* tic for plot */
    double *obs;
    gsl_rng_type *T;
    gsl_rng *r;

	printhead1();
    for (x = x0; x <= x1; x += h) {
        param[3] = x;

        GSL_SET_COMPLEX(&vsym, 0, 0);
        param[1] = 1.0; /* alpha = 1.0 */
        for (t = -R; t < R; t+= step)   /* pseudo numerical integration */
            vsym = gsl_complex_add(vsym, levy(&t, param));
        vsym = gsl_complex_div_real(vsym, (2.0 * M_PI));

        printf("%f\t%f\n", x, GSL_REAL(vsym)*step);
	}
	puts("e");
    for (x = x0; x <= x1; x += h) {
        param[3] = x;

        GSL_SET_COMPLEX(&vsym2, 0, 0);
        param[1] = 2.0; /* alpha = 2.0 */
        for (t = -R; t < R; t+= step)   /* pseudo numerical integration */
            vsym2 = gsl_complex_add(vsym2, levy(&t, param));
        vsym2 = gsl_complex_div_real(vsym2, (2.0 * M_PI));

        printf("%f\t%f\t%f\t%f\n", x, GSL_REAL(vsym2)*step);
	}
	puts("e");
	printhead2();
    for (x = x0; x <= x1; x += h) {
        param[3] = x;

        GSL_SET_COMPLEX(&vskew, 0, 0);
        param[1] = 1.0; /* alpha = 1.0 */
        for (t = -R; t < R; t+= step)   /* pseudo numerical integration */
            vskew = gsl_complex_add(vskew, skew(&t, param));
        vskew = gsl_complex_div_real(vskew, (2.0 * M_PI));

        printf("%f\t%f\t%f\t%f\n", x, GSL_REAL(vskew)*step);
    }

    return 0;
}

void printhead1(void)
{
	puts("set terminal epslatex");
	puts("set output 'levy.tex'");
	puts("set title 'Levy Distribution'");
	puts("set xlabel '$x$'");
	puts("set ylabel '$p(x)$'");
	puts("plot [-5:5][0:0.4] "
	     "'-' using 1:2 with lines title '$c = 1, \\alpha = 1.0$',"
	     "'-' using 1:3 with lines title '$c = 1, \\alpha = 2.0$'");
}
void printhead2()
{
	puts("set output 'levy_skew.tex'");
	puts("set title 'Levy Skew Distribution'");
	puts("set xlabel '$x$'");
	puts("set ylabel '$p(x)$'");
	puts("plot [-5:5][0:0.35] "
	     "'-' with lines title '$c = 1, \\alpha = 1.0, \\beta = 1.0$'");
}
