#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <gsl/gsl_bspline.h>
#include <gsl/gsl_multifit.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_statistics.h>

/* フィットするデータ点数 */
#define N 200

/* 係数の個数 */
#define NCOEFFS 12

/* k = 4 なので、nbreak = ncoeffs + 2 - k = ncoeffs - 2 */
#define NBREAK (NCOEFFS - 2)

int main (void)
{
        const size_t n = N;
        const size_t ncoeffs = NCOEFFS;
        const size_t nbreak = NBREAK;
        size_t i, j;
        gsl_bspline_workspace *bw;
        gsl_vector *B;
        double dy;
        gsl_rng *r;
        gsl_vector *c, *w;
        gsl_vector *x, *y;
        gsl_matrix *X, *cov;
        gsl_multifit_linear_workspace *mw;
        double chisq;
        double Rsq;
        double dof;

        gsl_rng_env_setup();
        r = gsl_rng_alloc(gsl_rng_default);

        /* 3 次の B スプラインのためん作業領域確保 (k = 4) */
        bw = gsl_bspline_alloc(4, nbreak);
        B = gsl_vector_alloc(ncoeffs);

        x = gsl_vector_alloc(n);
        y = gsl_vector_alloc(n);
        X = gsl_matrix_alloc(n, ncoeffs);
        c = gsl_vector_alloc(ncoeffs);
        w = gsl_vector_alloc(n);
        cov = gsl_matrix_alloc(ncoeffs, ncoeffs);
        mw = gsl_multifit_linear_alloc(n, ncoeffs);

        printf("#m=0,S=0\n");
        /* データ生成 */
        for (i = 0; i < n; ++i) {
                double sigma;
                double xi = (15.0 / (N - 1)) * i;
                double yi = cos(xi) * exp(-0.1 * xi);

                sigma = 0.1 * yi;
                dy = gsl_ran_gaussian(r, sigma);
                yi += dy;

                gsl_vector_set(x, i, xi);
                gsl_vector_set(y, i, yi);
                gsl_vector_set(w, i, 1.0 / (sigma * sigma));

                printf("%f %f\n", xi, yi);
        }

        /* フィッティングする区間は等間隔に分割する */
        gsl_bspline_knots_uniform(0.0, 15.0, bw);

        /* 各点における基底関数の係数による行列 X を求める*/
        for (i = 0; i < n; ++i) {
                double xi = gsl_vector_get(x, i);

                /* 各 j について B_j(xi) を計算する */
                gsl_bspline_eval(xi, B, bw);

                /* X の第 i 行を計算 */
                for (j = 0; j < ncoeffs; ++j) {
                        double Bj = gsl_vector_get(B, j);
                        gsl_matrix_set(X, i, j, Bj);
                }
        }

        /* フィットを行う */
        gsl_multifit_wlinear(X, w, y, c, cov, &chisq, mw);

        dof = n - ncoeffs;
        Rsq = 1.0 - chisq / gsl_stats_wtss(w->data, 1, y->data, 1, y->size);

        fprintf(stderr, "chisq/dof = %e, Rsq = %f\n", chisq / dof, Rsq);

        /* 滑らかになった曲線を出力 */
        double xi, yi, yerr;
        printf("#m=1,S=0\n");
        for (xi = 0.0; xi < 15.0; xi += 0.1) {
                gsl_bspline_eval(xi, B, bw);
                gsl_multifit_linear_est(B, c, cov, &yi, &yerr);
                printf("%f %f\n", xi, yi);
        }

        gsl_rng_free(r);
        gsl_bspline_free(bw);
        gsl_vector_free(B);
        gsl_vector_free(x);
        gsl_vector_free(y);
        gsl_matrix_free(X);
        gsl_vector_free(c);
        gsl_vector_free(w);
        gsl_matrix_free(cov);
        gsl_multifit_linear_free(mw);

        return 0;
} /* main() */

